import 'dart:async';

import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:permission_handler/permission_handler.dart';

enum CONNECTION_TYPE { WIFI, MOBILE }

class AndroidFlutterWifi {
  static const MethodChannel _channel = MethodChannel('android_flutter_wifi');
  static const _WIFI_INFO_CALL = "getWifiInfo";
  static const _WIFI_LIST_CALL = "getWifiList";
  static const _IS_CONNECTED_CALL = "isConnected";
  static const _IS_CONNECTION_FAST = "isConnectionFast";
  static const _CONNECTION_TYPE_CALL = "getConnectionType";
  static final _ENABLE_WIFI_CALL = "enableWifi";
  static final _DISABLE_WIFI_CALL = "disableWifi";
  static final _IS_WIFI_ENABLED_CALL = "isWifiEnabled";
  static final _GET_DHCP_INFO = "getDhcpInfo";
  static final _CONNECT_TO_NETWORK = "connectToNetwork";
  static final _CHECK_CONNECTING_NETWORK = "checkConnectingNetworkStatus";

  ///init method call location permission which is necessary
  ///get any information about Wi-FI in android
  static Future<bool?> init() async {
    var isPermissionGranted = await Permission.location.isGranted;
    if (!isPermissionGranted) {
      PermissionStatus permissionStatus = await Permission.location.request();
      if (permissionStatus.isGranted) {
        return true;
      }
    } else {
      return false;
    }
  }

  /// getWifiScanResult return wifi list
  static Future<List<WifiNetwork>> getWifiScanResult() async {
    List<dynamic> wifiMapList = await _channel.invokeMethod(_WIFI_LIST_CALL);
    List<WifiNetwork> wifiList =
        wifiMapList.map((e) => WifiNetwork.fromMap(e)).toList();
    return wifiList;
  }

  static Future<bool?> isConnectionFast() async {
    final bool isConnectionFast =
        await _channel.invokeMethod(_IS_CONNECTION_FAST);
    return isConnectionFast;
  }

  static Future<bool> isConnected() async {
    final bool isConnected = await _channel.invokeMethod(_IS_CONNECTED_CALL);
    return isConnected;
  }

  static Future<ActiveWifiNetwork> getActiveWifiInfo() async {
    Map<dynamic, dynamic> wifiObject =
        await _channel.invokeMethod(_WIFI_INFO_CALL);

    ActiveWifiNetwork activeWifiNetwork = ActiveWifiNetwork.fromMap(wifiObject);
    return activeWifiNetwork;
  }

  static Future<CONNECTION_TYPE> getConnectionType() async {
    String connection = await _channel.invokeMethod(_CONNECTION_TYPE_CALL);
    if (connection == 'wifi') {
      return CONNECTION_TYPE.WIFI;
    } else {
      return CONNECTION_TYPE.MOBILE;
    }
  }

  static Future<void> enableWifi() async {
    await _channel.invokeMethod(_ENABLE_WIFI_CALL);
  }

  static Future<void> disableWifi() async {
    await _channel.invokeMethod(_DISABLE_WIFI_CALL);
  }

  static Future<bool> isWifiEnabled() async {
    bool isWifiEnabled = await _channel.invokeMethod(_IS_WIFI_ENABLED_CALL);
    return isWifiEnabled;
  }

  static Future<DhcpInfo> getDhcpInfo() async {
    Map<dynamic, dynamic> map = await _channel.invokeMethod(_GET_DHCP_INFO);
    return DhcpInfo.fromMap(map);
  }

  /// Convert integer ipv4 address to understandable string like "127.0.0.1"
  static String toIp(String ip) {
    try {
      int value = int.parse(ip);
      return [
        (value >> 24) & 0xff,
        (value >> 16) & 0xff,
        (value >> 8) & 0xff,
        value & 0xff
      ].join('.');
    } on FormatException {
      // This error is thrown if int. Parse doesn't find an integer in the string given
      throw Exception(
          "This integer can't be proccess by int.parse, please check your value: $ip");
    }
  }

  static String getFormedIp(String ipAddress) {
    if (ipAddress.isEmpty) {
      debugPrint('Error: IP addess is empty');
      return '';
    } else if (ipAddress.contains('.')) {
      List<String> ipAddressList = ipAddress.split('.');
      String formedIp = ipAddressList.reversed.join('.').toString();
      return formedIp;
    } else {
      debugPrint('Error: Please pass an IP adrress');
      return '';
    }
  }

  static Future<bool> connectToNetwork(String ssid, String password) async {
    var args = {'ssid': ssid, 'password': password};
    var isConnected = await _channel.invokeMethod(_CONNECT_TO_NETWORK, args);
    await Future.delayed(const Duration(seconds: 10));
    var params = {'ssid': ssid};
    isConnected = await _channel.invokeMethod(_CHECK_CONNECTING_NETWORK, params);
    debugPrint('----------------IsConnected in android flutter wifi--------------: ${isConnected.toString()}');
    return isConnected;
    //_checkConnectingNetwork();
  }

  static Future<bool> _checkSsidConnection(String ssid) async{
    var params = {'ssid': ssid};
    var isConnected = await _channel.invokeMethod(_CHECK_CONNECTING_NETWORK, params);
    debugPrint('----------------IsConnected in android flutter wifi--------------: ${isConnected.toString()}');
    return isConnected;
  }
/*
  static Future<Map<String, dynamic>> _checkConnectingNetwork() async {
    Map<String, dynamic> map = {};
    var result = await _channel.invokeMethod(_CHECK_CONNECTING_NETWORK);
    if (result == null) {
      _checkConnectingNetwork();
    } else {
      map['success'] = result['success'];
      map['error'] = result['result'];
    }
    debugPrint('Result: ${result.toString()}');

    return map;
  }*/
}

class WifiNetwork {
  String? ssid;
  String? bssid;
  String? frequency;
  String? level;
  String? security;
  String? signalLevel;

  WifiNetwork(
      {this.ssid,
      this.bssid,
      this.frequency,
      this.level,
      this.security,
      this.signalLevel});

  factory WifiNetwork.fromMap(Map<dynamic, dynamic> map) {
    return WifiNetwork(
        bssid: map['bssid'],
        frequency: map['frequency'],
        level: map['level'],
        security: map['security'],
        ssid: map['ssid'],
        signalLevel: map['signal_level']);
  }
}

class ActiveWifiNetwork {
  String? ip;
  String? bssid;
  String? frequency;
  String? linkSpeed;
  String? networkId;
  String? ssid;

  ActiveWifiNetwork(
      {this.ip,
      this.bssid,
      this.frequency,
      this.linkSpeed,
      this.networkId,
      this.ssid});

  factory ActiveWifiNetwork.fromMap(Map<dynamic, dynamic> map) {
    return ActiveWifiNetwork(
        ssid: map['ssid'],
        frequency: map['frequency'],
        ip: map['ip'],
        linkSpeed: map['link_speed'],
        bssid: map['mac'],
        networkId: map['network_id']);
  }
}

class DhcpInfo {
  String? ip;
  String? gateway;
  String? serverAddress;
  String? dns1;
  String? dns2;
  String? leaseDuration;
  String? netMask;
  String? gatewayIp;

  DhcpInfo(
      {this.ip,
      this.gateway,
      this.serverAddress,
      this.dns1,
      this.dns2,
      this.leaseDuration,
      this.netMask,
      this.gatewayIp});

  factory DhcpInfo.fromMap(Map<dynamic, dynamic> map) {
    return DhcpInfo(
        dns1: map['dns_1'].toString(),
        dns2: map['dns_2'].toString(),
        gateway: map['gateway'].toString(),
        leaseDuration: map['lease_duration'].toString(),
        netMask: map['net_mask'].toString(),
        ip: map['ip_address'].toString(),
        serverAddress: map['server_address'].toString(),
        gatewayIp: map['gateway_ip'].toString().split('').reversed.join(''));
  }
}
